@@ -5,7 +5,7 @@
  * thread, and destroy a previously oppened video output thread.
  *****************************************************************************
  * Copyright (C) 2000 VideoLAN
- * $Id: video_output.c,v 1.131 2001/05/31 03:57:54 sam Exp $
+ * $Id: video_output.c,v 1.137 2001/08/14 00:00:04 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -196,7 +196,7 @@ vout_thread_t * vout_CreateThread   ( int *pi_status )
     p_vout->b_fullscreen          = main_GetIntVariable( VOUT_FULLSCREEN_VAR,
                                                      VOUT_FULLSCREEN_DEFAULT );
 
-    intf_WarnMsg( 3, "wished configuration: %dx%d, %d/%d bpp (%d Bpl)",
+    intf_WarnMsg( 3, "vout info: asking for %dx%d, %d/%d bpp (%d Bpl)",
                   p_vout->i_width, p_vout->i_height, p_vout->i_screen_depth,
                   p_vout->i_bytes_per_pixel * 8, p_vout->i_bytes_per_line );
 
@@ -212,6 +212,10 @@ vout_thread_t * vout_CreateThread   ( int *pi_status )
     /* Initialize buffer index */
     p_vout->i_buffer_index      = 0;
 
+    /* Initialize fonts */
+    p_vout->p_default_font = NULL;
+    p_vout->p_large_font = NULL;
+
     /* Initialize pictures and subpictures - translation tables and functions
      * will be initialized later in InitThread */
     for( i_index = 0; i_index < VOUT_MAX_PICTURES; i_index++)
@@ -237,17 +241,11 @@ vout_thread_t * vout_CreateThread   ( int *pi_status )
                            (void *) RunThread, (void *) p_vout) )
     {
         intf_ErrMsg("vout error: %s", strerror(ENOMEM));
-        vout_UnloadFont( p_vout->p_default_font );
-        vout_UnloadFont( p_vout->p_large_font );
-        p_vout->pf_destroy( p_vout );
+        module_Unneed( p_vout->p_module );
         free( p_vout );
         return( NULL );
     }
 
-    intf_WarnMsg( 1, "vout: video display initialized (%dx%d, %d/%d bpp)",
-              p_vout->i_width, p_vout->i_height, p_vout->i_screen_depth,
-              p_vout->i_bytes_per_pixel * 8 );
-
     /* If status is NULL, wait until the thread is created */
     if( pi_status == NULL )
     {
@@ -687,9 +685,6 @@ picture_t *vout_CreatePicture( vout_thread_t *p_vout, int i_type,
 #endif
         vlc_mutex_unlock( &p_vout->picture_lock );
 
-        /* Initialize mutex */
-        vlc_mutex_init( &(p_free_picture->lock_deccount) );
-
         return( p_free_picture );
     }
 
@@ -727,9 +722,6 @@ void vout_DestroyPicture( vout_thread_t *p_vout, picture_t *p_pic )
     intf_DbgMsg("picture %p", p_pic);
 #endif
 
-    /* destroy the lock that had been initialized in CreatePicture */
-    vlc_mutex_destroy( &(p_pic->lock_deccount) );
-
     vlc_mutex_unlock( &p_vout->picture_lock );
 }
 
@@ -880,12 +872,18 @@ static int InitThread( vout_thread_t *p_vout )
      * own error messages */
     if( p_vout->pf_create( p_vout ) )
     {
-        module_Unneed( p_vout->p_module );
-        free( p_vout );
+        /* If pf_create has failed then we have to make sure
+         * pf_destroy won't be called, because the plugin should have
+         * cleaned up all its mess */
+        p_vout->pf_destroy = NULL;
         return( 1 );
     }
 
-    intf_WarnMsg( 3, "actual configuration: %dx%d, %d/%d bpp (%d Bpl), "
+    intf_WarnMsg( 1, "vout: video display initialized (%dx%d, %d/%d bpp)",
+              p_vout->i_width, p_vout->i_height, p_vout->i_screen_depth,
+              p_vout->i_bytes_per_pixel * 8 );
+
+    intf_WarnMsg( 3, "vout info: got %dx%d, %d/%d bpp (%d Bpl), "
                   "masks: 0x%x/0x%x/0x%x",
                   p_vout->i_width, p_vout->i_height, p_vout->i_screen_depth,
                   p_vout->i_bytes_per_pixel * 8, p_vout->i_bytes_per_line,
@@ -920,9 +918,13 @@ static int InitThread( vout_thread_t *p_vout )
         intf_ErrMsg( "vout error: could not load large font" );
     }
 
-    /* Initialize output method - this function issues its own error messages */
+    /* Initialize output method. This function issues its own error messages */
     if( p_vout->pf_init( p_vout ) )
     {
+        /* If pf_init has failed then we have to make sure
+         * pf_destroy won't be called, because the plugin should have
+         * cleaned up all its mess */
+        p_vout->pf_destroy = NULL;
         return( 1 );
     }
 
@@ -930,6 +932,9 @@ static int InitThread( vout_thread_t *p_vout )
     if( vout_InitYUV( p_vout ) )
     {
         intf_ErrMsg("vout error: can't allocate YUV translation tables");
+        p_vout->pf_destroy( p_vout );
+        /* Make sure pf_destroy won't be called again */
+        p_vout->pf_destroy = NULL;
         return( 1 );
     }
 
@@ -965,13 +970,12 @@ static void RunThread( vout_thread_t *p_vout)
     /*
      * Initialize thread
      */
-    p_vout->b_error = InitThread( p_vout );
-    if( p_vout->b_error )
+    if( InitThread( p_vout ) )
     {
+        /* Something bad happened */
         DestroyThread( p_vout, THREAD_ERROR );
         return;
     }
-
     /*
      * Main loop - it is not executed if an error occured during
      * initialization
@@ -979,6 +983,7 @@ static void RunThread( vout_thread_t *p_vout)
     while( (!p_vout->b_die) && (!p_vout->b_error) )
     {
         /* Initialize loop variables */
+        p_vout->p_rendered_pic = NULL;
         p_pic =         NULL;
         p_subpic =      NULL;
         p_ephemer =     NULL;
@@ -1030,7 +1035,7 @@ static void RunThread( vout_thread_t *p_vout)
                     p_vout->i_pictures--;
                 }
                 intf_WarnMsg( 1,
-                        "warning: late picture skipped (%p)", p_pic );
+                        "vout warning: late picture skipped (%p)", p_pic );
                 vlc_mutex_unlock( &p_vout->picture_lock );
 
                 continue;
@@ -1154,14 +1159,14 @@ static void RunThread( vout_thread_t *p_vout)
             p_vout->last_display_date = display_date;
             p_vout->p_rendered_pic = p_pic;
 
-            /* Set picture dimensions and clear buffer */
-            SetBufferPicture( p_vout, p_pic );
-
             /* FIXME: if b_need_render == 0 we need to do something with
              * the subpictures one day. */
 
             if( p_vout->b_need_render && b_display )
             {
+                /* Set picture dimensions and clear buffer */
+                SetBufferPicture( p_vout, p_pic );
+
                 /* Render picture and information */
                 RenderPicture( p_vout, p_pic );
                 if( p_vout->b_info )
@@ -1276,7 +1281,7 @@ static void RunThread( vout_thread_t *p_vout)
          */
         if( p_vout->pf_manage( p_vout ) | Manage( p_vout ) )
         {
-            /* A fatal error occured, and the thread must terminate immediately,
+            /* A fatal error occured, and the thread must terminate immediately
              * without displaying anything - setting b_error to 1 cause the
              * immediate end of the main while() loop. */
             p_vout->b_error = 1;
@@ -1378,7 +1383,7 @@ static void DestroyThread( vout_thread_t *p_vout, int i_status )
     /* Destroy thread structures allocated by Create and InitThread */
     vout_UnloadFont( p_vout->p_default_font );
     vout_UnloadFont( p_vout->p_large_font );
-    p_vout->pf_destroy( p_vout );
+    if( p_vout->pf_destroy != NULL ) p_vout->pf_destroy( p_vout );
 
     /* Destroy the locks */
     vlc_mutex_destroy( &p_vout->picture_lock );
@@ -1996,11 +2001,17 @@ static void RenderSubPicture( vout_thread_t *p_vout, picture_t *p_pic,
         switch( p_subpic->i_type )
         {
         case DVD_SUBPICTURE:                          /* DVD subpicture unit */
-            vout_RenderRGBSPU( p_pic, p_subpic,
-                               &p_vout->p_buffer[ p_vout->i_buffer_index ],
-                               p_vout->i_bytes_per_pixel,
-                               p_vout->i_bytes_per_line );
-            /* vout_RenderYUVSPU( p_pic, p_subpic ); */
+            if( p_vout->b_need_render )
+            {
+                vout_RenderRGBSPU( p_pic, p_subpic,
+                                   &p_vout->p_buffer[ p_vout->i_buffer_index ],
+                                   p_vout->i_bytes_per_pixel,
+                                   p_vout->i_bytes_per_line );
+            }
+            else
+            {
+                vout_RenderYUVSPU( p_pic, p_subpic );
+            }
             break;
 
         case TEXT_SUBPICTURE:                            /* single line text */
